@@ -8,9 +8,9 @@
     require DBI::Gofer::Response;
     require Carp;
 
-    our $VERSION = "0.015327";
+    our $VERSION = sprintf("0.%06d", q$Revision: 9478 $ =~ /(\d+)/o);
 
-#   $Id: Gofer.pm 15326 2012-06-06 16:32:38Z Tim $
+#   $Id: Gofer.pm 9478 2007-05-01 11:03:38Z timbo $
 #
 #   Copyright (c) 2007, Tim Bunce, Ireland
 #
@@ -25,11 +25,10 @@
         CachedKids
         Callbacks
         DbTypeSubclass
-        ErrCount ErrChangeCount Executed
+        ErrCount Executed
         FetchHashKeyName
         HandleError HandleSetErr
         InactiveDestroy
-        AutoInactiveDestroy
         PrintError PrintWarn
         Profile
         RaiseError
@@ -47,7 +46,7 @@
         dbi_connect_method
     );
 
-    our $drh = undef;    # holds driver handle once initialized
+    our $drh = undef;    # holds driver handle once initialised
     our $methods_already_installed;
 
     sub driver{
@@ -56,12 +55,9 @@
         DBI->setup_driver('DBD::Gofer');
 
         unless ($methods_already_installed++) {
-            my $opts = { O=> 0x0004 }; # IMA_KEEP_ERR
-            DBD::Gofer::db->install_method('go_dbh_method', $opts);
-            DBD::Gofer::st->install_method('go_sth_method', $opts);
-            DBD::Gofer::st->install_method('go_clone_sth',  $opts);
-            DBD::Gofer::db->install_method('go_cache',      $opts);
-            DBD::Gofer::st->install_method('go_cache',      $opts);
+            DBD::Gofer::db->install_method('go_dbh_method', { O=> 0x0004 }); # IMA_KEEP_ERR
+            DBD::Gofer::st->install_method('go_sth_method', { O=> 0x0004 }); # IMA_KEEP_ERR
+            DBD::Gofer::st->install_method('go_clone_sth',  { O=> 0x0004 }); # IMA_KEEP_ERR
         }
 
         my($class, $attr) = @_;
@@ -81,28 +77,12 @@
     }
 
 
-    sub go_cache {
-        my $h = shift;
-        $h->{go_cache} = shift if @_;
-        # return handle's override go_cache, if it has one
-        return $h->{go_cache} if defined $h->{go_cache};
-        # or else the transports default go_cache
-        return $h->{go_transport}->{go_cache};
-    }
-
-
     sub set_err_from_response { # set error/warn/info and propagate warnings
-        my $h = shift;
-        my $response = shift;
+        my ($h, $response) = @_;
         if (my $warnings = $response->warnings) {
             warn $_ for @$warnings;
         }
-        my ($err, $errstr, $state) = $response->err_errstr_state;
-        # Only set_err() if there's an error else leave the current values
-        # (The current values will normally be set undef by the DBI dispatcher
-        # except for methods marked KEEPERR such as ping.)
-        $h->set_err($err, $errstr, $state) if defined $err;
-        return undef;
+        return $h->set_err($response->err, $response->errstr, $response->state);
     }
 
 
@@ -126,7 +106,7 @@
                 $sub = sub { return shift->go_dbh_method(undef, $method, @_) };
             }
             else {
-                $sub = sub { shift->set_err($DBI::stderr, "Can't call \$${type}h->$method when using DBD::Gofer"); return; };
+                $sub = sub { shift->set_err(1, "Can't call \$${type}h->$method when using DBD::Gofer"); return; };
             }
             no strict 'refs';
             *$driver_method = $sub;
@@ -156,7 +136,7 @@
 
         # first remove dsn= and everything after it
         my $remote_dsn = ($dsn =~ s/;?\bdsn=(.*)$// && $1)
-            or return $drh->set_err($DBI::stderr, "No dsn= argument in '$orig_dsn'");
+            or return $drh->set_err(1, "No dsn= argument in '$orig_dsn'");
 
         if ($attr->{go_bypass}) { # don't use DBD::Gofer for this connection
             # useful for testing with DBI_AUTOPROXY, e.g., t/03handle.t
@@ -179,34 +159,25 @@
             $policy_class = "DBD::Gofer::Policy::$policy_class"
                 unless $policy_class =~ /::/;
             _load_class($policy_class)
-                or return $drh->set_err($DBI::stderr, "Can't load $policy_class: $@");
+                or return $drh->set_err(1, "Can't load $policy_class: $@");
             # replace policy name in %go_attr with policy object
             $go_attr{go_policy} = eval { $policy_class->new(\%go_attr) }
-                or return $drh->set_err($DBI::stderr, "Can't instanciate $policy_class: $@");
+                or return $drh->set_err(1, "Can't instanciate $policy_class: $@");
         }
         # policy object is left in $go_attr{go_policy} so transport can see it
         my $go_policy = $go_attr{go_policy};
 
-        if ($go_attr{go_cache} and not ref $go_attr{go_cache}) { # if not a cache object already
-            my $cache_class = $go_attr{go_cache};
-            $cache_class = "DBI::Util::CacheMemory" if $cache_class eq '1';
-            _load_class($cache_class)
-                or return $drh->set_err($DBI::stderr, "Can't load $cache_class $@");
-            $go_attr{go_cache} = eval { $cache_class->new() }
-                or $drh->set_err(0, "Can't instanciate $cache_class: $@"); # warning
-        }
-
-        # delete any other attributes that don't apply to transport
+        # but delete any other attributes that don't appy to transport
         my $go_connect_method = delete $go_attr{go_connect_method};
 
         my $transport_class = delete $go_attr{go_transport}
-            or return $drh->set_err($DBI::stderr, "No transport= argument in '$orig_dsn'");
+            or return $drh->set_err(1, "No transport= argument in '$orig_dsn'");
         $transport_class = "DBD::Gofer::Transport::$transport_class"
             unless $transport_class =~ /::/;
         _load_class($transport_class)
-            or return $drh->set_err($DBI::stderr, "Can't load $transport_class: $@");
+            or return $drh->set_err(1, "Can't load $transport_class: $@");
         my $go_transport = eval { $transport_class->new(\%go_attr) }
-            or return $drh->set_err($DBI::stderr, "Can't instanciate $transport_class: $@");
+            or return $drh->set_err(1, "Can't instanciate $transport_class: $@");
 
         my $request_class = "DBI::Gofer::Request";
         my $go_request = eval {
@@ -225,7 +196,7 @@
             $request_class->new({
                 dbh_connect_call => [ $go_connect_method, $remote_dsn, $user, $auth, $go_attr ],
             })
-        } or return $drh->set_err($DBI::stderr, "Can't instanciate $request_class: $@");
+        } or return $drh->set_err(1, "Can't instanciate $request_class: $@");
 
         my ($dbh, $dbh_inner) = DBI::_new_dbh($drh, {
             'Name' => $dsn,
@@ -244,7 +215,7 @@
         if (not $skip_connect_check) {
             if (not $dbh->go_dbh_method(undef, 'ping')) {
                 return undef if $dbh->err; # error already recorded, typically
-                return $dbh->set_err($DBI::stderr, "ping failed");
+                return $dbh->set_err(1, "ping failed");
             }
         }
 
@@ -293,10 +264,7 @@
             if $meta->{go_last_insert_id_args};
 
         my $transport = $dbh->{go_transport}
-            or return $dbh->set_err($DBI::stderr, "Not connected (no transport)");
-
-        local $transport->{go_cache} = $dbh->{go_cache}
-            if defined $dbh->{go_cache};
+            or return $dbh->set_err(1, "Not connected (no transport)");
 
         my ($response, $retransmit_sub) = $transport->transmit_request($request);
         $response ||= $transport->receive_response($request, $retransmit_sub);
@@ -308,7 +276,7 @@
 
         if (my $dbh_attributes = $response->dbh_attributes) {
 
-            # XXX installed_methods piggybacks on dbh_attributes for now
+            # XXX installed_methods piggbacks on dbh_attributes for now
             if (my $installed_methods = delete $dbh_attributes->{dbi_installed_methods}) {
                 DBD::Gofer::install_methods_proxy($installed_methods)
                     if $dbh->{go_request_count}==1;
@@ -333,10 +301,6 @@
             # and return that new sth as if it came from original request
             $rv = [ $sth ];
         }
-        elsif (!$rv) { # should only occur for major transport-level error
-            #carp("no rv in response { @{[ %$response ]} }");
-            $rv = [ ];
-        }
 
         DBD::Gofer::set_err_from_response($dbh, $response);
 
@@ -368,7 +332,7 @@
             my $cache;
             my $cache_key;
             if (my $cache_it = $dbh->{go_policy}->$policy_name(undef, $dbh, @_)) {
-                $cache = $dbh->{go_meta_cache} ||= {}; # keep separate from go_cache
+                $cache = $dbh->{go_cache} ||= {};
                 $cache_key = sprintf "%s_wa%d(%s)", $policy_name, wantarray||0,
                     join(",\t", map { # XXX basic but sufficient for now
                          !ref($_)            ? DBI::neat($_,1e6)
@@ -444,7 +408,7 @@
         begin_work commit rollback
     )) {
         no strict 'refs';
-        *$method = sub { return shift->set_err($DBI::stderr, "$method not available with DBD::Gofer") }
+        *$method = sub { return shift->set_err(1, "$method not available with DBD::Gofer") }
     }
 
 
@@ -514,7 +478,7 @@
 
         # dbh attributes are set at connect-time - see connect()
         carp("Can't alter \$dbh->{$attrib} after handle created with DBD::Gofer") if $dbh->FETCH('Warn');
-        return $dbh->set_err($DBI::stderr, "Can't alter \$dbh->{$attrib} after handle created with DBD::Gofer");
+        return $dbh->set_err(1, "Can't alter \$dbh->{$attrib} after handle created with DBD::Gofer");
     }
 
     sub disconnect {
@@ -526,7 +490,7 @@
     sub prepare {
         my ($dbh, $statement, $attr)= @_;
 
-        return $dbh->set_err($DBI::stderr, "Can't prepare when disconnected")
+        return $dbh->set_err(1, "Can't prepare when disconnected")
             unless $dbh->FETCH('Active');
 
         $attr = { %$attr } if $attr; # copy so we can edit
@@ -537,7 +501,6 @@
                       || $dbh->{go_prepare_method}
                       || $policy->prepare_method($dbh, $statement, $attr)
                       || 'prepare'; # e.g. for code not using placeholders
-        my $go_cache = delete $attr->{go_cache};
         # set to undef if there are no attributes left for the actual prepare call
         $attr = undef if $attr and not %$attr;
 
@@ -549,7 +512,6 @@
             go_transport => $dbh->{go_transport},
             go_policy => $policy,
             go_last_insert_id_args => $lii_args,
-            go_cache => $go_cache,
         });
         $sth->STORE(Active => 0);
 
@@ -570,7 +532,6 @@
         }, $if_active);
     }
 
-    *go_cache = \&DBD::Gofer::go_cache;
 }
 
 
@@ -589,7 +550,7 @@
             # but only works properly for params 1..9
             # (reverse because of the unshift)
             my @params = reverse sort keys %$ParamValues;
-            if (@params > 9 && ($sth->{Database}{go_dsn}||'') =~ /dbi:Sybase/) {
+            if (@params > 9 && $sth->{Database}{go_dsn} =~ /dbi:Sybase/) {
                 # if more than 9 then we need to do a proper numeric sort
                 # also warn to alert user of this issue
                 warn "Sybase param binding order hack in use";
@@ -621,10 +582,7 @@
             or $dbh->{go_request_count}==1;
 
         my $transport = $sth->{go_transport}
-            or return $sth->set_err($DBI::stderr, "Not connected (no transport)");
-
-        local $transport->{go_cache} = $sth->{go_cache}
-            if defined $sth->{go_cache};
+            or return $sth->set_err(1, "Not connected (no transport)");
 
         my ($response, $retransmit_sub) = $transport->transmit_request($request);
         $response ||= $transport->receive_response($request, $retransmit_sub);
@@ -641,19 +599,19 @@
             $dbh->{go_dbh_attributes_fetched} = $dbh_attributes;
         }
 
-        my $rv = $response->rv; # may be undef on error
+        my $rv = $response->rv;
         if ($response->sth_resultsets) {
             # setup first resultset - including sth attributes
             $sth->more_results;
         }
         else {
             $sth->STORE(Active => 0);
-            $sth->{go_rows} = $rv;
+            $sth->{go_rows} = $response->rv;
         }
         # set error/warn/info (after more_results as that'll clear err)
         DBD::Gofer::set_err_from_response($sth, $response);
 
-        return $rv;
+        return $response->rv;
     }
 
 
@@ -687,13 +645,13 @@
         };
 
         my $resultset_list = $response->sth_resultsets
-            or return $sth->set_err($DBI::stderr, "No sth_resultsets");
+            or return $sth->set_err(1, "No sth_resultsets");
 
         my $meta = shift @$resultset_list
             or return undef; # no more result sets
         #warn "more_results: ".Data::Dumper::Dumper($meta);
 
-        # pull out the special non-attributes first
+        # pull out the special non-atributes first
         my ($rowset, $err, $errstr, $state)
             = delete @{$meta}{qw(rowset err errstr state)};
 
@@ -788,7 +746,7 @@
         # do the store locally anyway, just in case
         $sth->SUPER::STORE($attrib => $value);
 
-        return $sth->set_err($DBI::stderr, $msg);
+        return $sth->set_err(1, $msg);
     }
 
     # sub bind_param_array
@@ -803,7 +761,6 @@
         return $sth->go_sth_method($attr);
     }
 
-    *go_cache = \&DBD::Gofer::go_cache;
 }
 
 1;
@@ -838,12 +795,10 @@ The C<...> represents attributes that influence the operation of the Gofer
 driver or transport. These are described below or in the documentation of the
 transport module being used.
 
-=encoding ISO8859-1
-
 =head1 DESCRIPTION
 
 DBD::Gofer is a DBI database driver that forwards requests to another DBI
-driver, usually in a separate process, often on a separate machine. It tries to
+driver, usually in a seperate process, often on a separate machine. It tries to
 be as transparent as possible so it appears that you are using the remote
 driver directly.
 
@@ -857,8 +812,6 @@ single response that includes all the data.
 This is very similar to the way http works as a stateless protocol for the web.
 Each request from your web browser can be handled by a different web server process.
 
-=head2 Use Cases
-
 This may seem like pointless overhead but there are situations where this is a
 very good thing. Let's consider a specific case.
 
@@ -875,8 +828,6 @@ results, such as $sth->{NAME}. This response is sent back to DBD::Gofer which
 unpacks it and presents it to the application as if it had executed the
 statement itself.
 
-=head2 Advantages
-
 Okay, but you still don't see the point? Well let's consider what we've gained:
 
 =head3 Connection Pooling and Throttling
@@ -894,8 +845,7 @@ DBD::Gofer to connect them to your smaller pool of 'dbi execute' web servers ins
 
 =head3 Caching
 
-Client-side caching is as simple as adding "C<cache=1>" to the DSN.
-This feature alone can be worth using DBD::Gofer for.
+Not yet implemented, but the single request-response architecture lends itself to caching.
 
 =head3 Fewer Network Round-trips
 
@@ -908,7 +858,7 @@ You no longer need drivers for your database on every system.  DBD::Gofer is pur
 =head1 CONSTRAINTS
 
 There are some natural constraints imposed by the DBD::Gofer 'stateless' approach.
-But not many:
+But not too many:
 
 =head2 You can't change database handle attributes after connect()
 
@@ -923,7 +873,7 @@ They can be set after connect() but the change is only applied locally.
 
 =head2 You can't change statement handle attributes after prepare()
 
-You can't change statement handle attributes after prepare.
+You can't change statment handle attributes after prepare.
 
 An exception is made for attributes with names starting "C<private_>":
 They can be set after prepare() but the change is only applied locally.
@@ -932,30 +882,29 @@ They can be set after prepare() but the change is only applied locally.
 
 AutoCommit only. Transactions aren't supported.
 
-(In theory transactions could be supported when using a transport that
-maintains a connection, like C<stream> does. If you're interested in this
-please get in touch via dbi-dev@perl.org)
-
 =head2 You can't call driver-private sth methods
 
 But that's rarely needed anyway.
 
+=head2 Per-row driver-private sth attributes aren't supported
+
+Some drivers provide sth attributes that relate to the row that was just
+fetched (e.g., Sybase and syb_result_type). These aren't supported.
+
 =head1 GENERAL CAVEATS
 
 A few important things to keep in mind when using DBD::Gofer:
 
-=head2 Temporary tables, locks, and other per-connection persistent state
-
-You shouldn't expect any per-session state to persist between requests.
-This includes locks and temporary tables.
+=head2 You shouldn't use temporary tables, locks, or other per-connection persistent state
 
 Because the server-side may execute your requests via a different
 database connections, you can't rely on any per-connection persistent state,
 such as temporary tables, being available from one request to the next.
 
-This is an easy trap to fall into. A good way to check for this is to test your
-code with a Gofer policy package that sets the C<connect_method> policy to
-'connect' to force a new connection for each request. The C<pedantic> policy does this.
+This is an easy trap to fall into and a difficult one to debug.
+The pipeone transport may help as it forces a new connection for each request.
+(It is very slow though, so I plan to add a way for the stream driver to use
+connect instead of connect_cache to achive the same effect much more efficiently.)
 
 =head2 Driver-private Database Handle Attributes
 
@@ -966,7 +915,7 @@ implemented the private_attribute_info() method (added in DBI 1.54).
 
 Driver-private sth attributes can be set in the prepare() call. TODO
 
-Some driver-private sth attributes may not be available if the driver has not
+Some driver-private dbh attributes may not be available if the driver has not
 implemented the private_attribute_info() method (added in DBI 1.54).
 
 =head2 Multiple Resultsets
@@ -983,7 +932,7 @@ more general mechanism is needed for other drivers to use.
 
 =head2 Methods that report an error always return undef
 
-With DBD::Gofer, a method that sets an error always return an undef or empty list.
+With DBD::Gofer a method that sets an error always return an undef or empty list.
 That shouldn't be a problem in practice because the DBI doesn't define any
 methods that return meaningful values while also reporting an error.
 
@@ -1012,8 +961,7 @@ last_insert_id() method.
 
 The array methods bind_param_array() and execute_array() are supported.
 When execute_array() is called the data is serialized and executed in a single
-round-trip to the Gofer server. This makes it very fast, but requires enough
-memory to store all the serialized data.
+round-trip to the Gofer server.
 
 The execute_for_fetch() method currently isn't optimised, it uses the DBI
 fallback behaviour of executing each tuple individually.
@@ -1047,8 +995,8 @@ a data structure before passing it to DBI::Gofer::Execute to execute. The same
 freeze and thaw is applied to the results.
 
 The null transport is the best way to test if your application will work with Gofer.
-Just set the DBI_AUTOPROXY environment variable to "C<dbi:Gofer:transport=null;policy=pedantic>"
-(see L</Using DBI_AUTOPROXY> below) and run your application, or ideally its test suite, as usual.
+Just set the DBI_AUTOPROXY environment variable to "C<dbi:Gofer:transport=null>"
+(see L</DBI_AUTOPROXY> below) and run your application, or ideally its test suite, as usual.
 
 It doesn't take any parameters.
 
@@ -1058,40 +1006,45 @@ The pipeone transport launches a subprocess for each request. It passes in the
 request and reads the response.
 
 The fact that a new subprocess is started for each request ensures that the
-server side is truly stateless. While this does make the transport I<very> slow,
-it is useful as a way to test that your application doesn't depend on
+server side is truly stateless. While this does make the transport very slow it
+is useful as a way to test that your application doesn't depend on
 per-connection state, such as temporary tables, persisting between requests.
 
 It's also useful both as a proof of concept and as a base class for the stream
 driver.
 
+This transport supports a timeout parameter in the dsn which specifies
+the maximum time it can take to send a requestor receive a response.
+
 =head3 stream
 
 The stream driver also launches a subprocess and writes requests and reads
 responses, like the pipeone transport.  In this case, however, the subprocess
-is expected to handle more that one request. (Though it will be automatically
-restarted if it exits.)
+is expected to handle more that one request. (Though it will be restarted if it exits.)
 
 This is the first transport that is truly useful because it can launch the
-subprocess on a remote machine using C<ssh>. This means you can now use DBD::Gofer
+subprocess on a remote machine using ssh. This means you can now use DBD::Gofer
 to easily access any databases that's accessible from any system you can login to.
 You also get all the benefits of ssh, including encryption and optional compression.
 
-See L</Using DBI_AUTOPROXY> below for an example.
+This transport supports a timeout parameter in the dsn which specifies the
+maximum time it can take to send a requestor receive a response.
+
+See L</DBI_AUTOPROXY> below for an example.
 
 =head2 Other Transports
 
-Implementing a Gofer transport is I<very> simple, and more transports are very welcome.
-Just take a look at any existing transports that are similar to your needs.
-
 =head3 http
 
-See the GoferTransport-http distribution on CPAN: http://search.cpan.org/dist/GoferTransport-http/
+See the GoferTransport-http distribution on CPAN.
 
 =head3 Gearman
 
-I know Ask Bjrn Hansen has implemented a transport for the C<gearman> distributed
-job system, though it's not on CPAN at the time of writing this.
+I know Ask Bjrn Hansen has implemented a transport for the gearman distributed
+job system. (Not yet on CPAN.)
+
+Implementing a transport is very simple, and more transports are very welcome.
+Just take a look at any existing transports that are similar to your needs.
 
 =head1 CONNECTING
 
@@ -1100,143 +1053,40 @@ where $transport is the name of the Gofer transport you want to use (see L</TRAN
 The C<transport> and C<dsn> attributes must be specified and the C<dsn> attributes must be last.
 
 Other attributes can be specified in the DSN to configure DBD::Gofer and/or the
-Gofer transport module being used. The main attributes after C<transport>, are
-C<url> and C<policy>. These and other attributes are described below.
+Gofer transport module being used. The two main attributes after C<transport>,
+are C<url> and C<policy>. These are described below.
 
 =head2 Using DBI_AUTOPROXY
 
 The simplest way to try out DBD::Gofer is to set the DBI_AUTOPROXY environment variable.
 In this case you don't include the C<dsn=> part. For example:
 
-    export DBI_AUTOPROXY="dbi:Gofer:transport=null"
+    export DBI_AUTOPROXY=dbi:Gofer:transport=null
 
 or, for a more useful example, try:
 
-    export DBI_AUTOPROXY="dbi:Gofer:transport=stream;url=ssh:user@example.com"
-
-=head2 Connection Attributes
-
-These attributes can be specified in the DSN. They can also be passed in the
-\%attr parameter of the DBI connect method by adding a "C<go_>" prefix to the name.
-
-=head3 transport
-
-Specifies the Gofer transport class to use. Required. See L</TRANSPORTS> above.
-
-If the value does not include C<::> then "C<DBD::Gofer::Transport::>" is prefixed.
-
-The transport object can be accessed via $h->{go_transport}.
-
-=head3 dsn
-
-Specifies the DSN for the remote side to connect to. Required, and must be last.
-
-=head3 url
-
-Used to tell the transport where to connect to. The exact form of the value depends on the transport used.
-
-=head3 policy
-
-Specifies the policy to use. See L</CONFIGURING BEHAVIOUR POLICY>.
-
-If the value does not include C<::> then "C<DBD::Gofer::Policy>" is prefixed.
-
-The policy object can be accessed via $h->{go_policy}.
-
-=head3 timeout
-
-Specifies a timeout, in seconds, to use when waiting for responses from the server side.
-
-=head3 retry_limit
-
-Specifies the number of times a failed request will be retried. Default is 0.
-
-=head3 retry_hook
-
-Specifies a code reference to be called to decide if a failed request should be retried.
-The code reference is called like this:
-
-  $transport = $h->{go_transport};
-  $retry = $transport->go_retry_hook->($request, $response, $transport);
-
-If it returns true then the request will be retried, up to the C<retry_limit>.
-If it returns a false but defined value then the request will not be retried.
-If it returns undef then the default behaviour will be used, as if C<retry_hook>
-had not been specified.
-
-The default behaviour is to retry requests where $request->is_idempotent is true,
-or the error message matches C</induced by DBI_GOFER_RANDOM/>.
-
-=head3 cache
-
-Specifies that client-side caching should be performed.  The value is the name
-of a cache class to use.
-
-Any class implementing get($key) and set($key, $value) methods can be used.
-That includes a great many powerful caching classes on CPAN, including the
-Cache and Cache::Cache distributions.
-
-You can use "C<cache=1>" is a shortcut for "C<cache=DBI::Util::CacheMemory>".
-See L<DBI::Util::CacheMemory> for a description of this simple fast default cache.
-
-The cache object can be accessed via $h->go_cache. For example:
-
-    $dbh->go_cache->clear; # free up memory being used by the cache
-
-The cache keys are the frozen (serialized) requests, and the values are the
-frozen responses.
-
-The default behaviour is to only use the cache for requests where
-$request->is_idempotent is true (i.e., the dbh has the ReadOnly attribute set
-or the SQL statement is obviously a SELECT without a FOR UPDATE clause.)
-
-For even more control you can use the C<go_cache> attribute to pass in an
-instantiated cache object. Individual methods, including prepare(), can also
-specify alternative caches via the C<go_cache> attribute. For example, to
-specify no caching for a particular query, you could use
-
-    $sth = $dbh->prepare( $sql, { go_cache => 0 } );
-
-This can be used to implement different caching policies for different statements.
-
-It's interesting to note that DBD::Gofer can be used to add client-side caching
-to any (gofer compatible) application, with no code changes and no need for a
-gofer server.  Just set the DBI_AUTOPROXY environment variable like this:
-
-    DBI_AUTOPROXY='dbi:Gofer:transport=null;cache=1'
-
-=head1 CONFIGURING BEHAVIOUR POLICY
-
-DBD::Gofer supports a 'policy' mechanism that allows you to fine-tune the number of round-trips to the Gofer server.
-The policies are grouped into classes (which may be subclassed) and referenced by the name of the class.
-
-The L<DBD::Gofer::Policy::Base> class is the base class for all the policy
-packages and describes all the available policies.
-
-Three policy packages are supplied with DBD::Gofer:
-
-L<DBD::Gofer::Policy::pedantic> is most 'transparent' but slowest because it
-makes more  round-trips to the Gofer server.
-
-L<DBD::Gofer::Policy::classic> is a reasonable compromise - it's the default policy.
-
-L<DBD::Gofer::Policy::rush> is fastest, but may require code changes in your applications.
-
-Generally the default C<classic> policy is fine. When first testing an existing
-application with Gofer it is a good idea to start with the C<pedantic> policy
-first and then switch to C<classic> or a custom policy, for final testing.
+    export DBI_AUTOPROXY=dbi:Gofer:transport=stream;url=ssh:user@example.com
 
 
-=head1 AUTHOR
+=head1 CONFIGURING VIA POLICY
 
-Tim Bunce, L<http://www.tim.bunce.name>
+XXX
 
-=head1 LICENCE AND COPYRIGHT
+policy=pedantic is most transparent but slow
 
-Copyright (c) 2007, Tim Bunce, Ireland. All rights reserved.
+policy=classic is a reasonable compromise, and is the default
 
-This module is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself. See L<perlartistic>.
+policy=rush is fastest but may require code changes in application
+
+See L<DBD::Gofer::Policy::Base> for more information.
+
+=head1 AUTHOR AND COPYRIGHT
+
+The DBD::Gofer, DBD::Gofer::* and DBI::Gofer::* modules are
+Copyright (c) 2007 Tim Bunce. Ireland.  All rights reserved.
+
+You may distribute under the terms of either the GNU General Public License or
+the Artistic License, as specified in the Perl README file.
 
 =head1 ACKNOWLEDGEMENTS
 
@@ -1255,11 +1105,11 @@ L<DBI>
 
 This section aims to record issues to be aware of when using Gofer with specific drivers.
 It usually only documents issues that are not natural consequences of the limitations
-of the Gofer approach - as documented above.
+of the Gofer approach - as documented avove.
 
 =head1 TODO
 
-This is just a random brain dump... (There's more in the source of the Changes file, not the pod)
+This is just a random brain dump... (There's more in Changes)
 
 Document policy mechanism
 
@@ -1267,6 +1117,8 @@ Add mechanism for transports to list config params and for Gofer to apply any th
 
 Driver-private sth attributes - set via prepare() - change DBI spec
 
+Caching of get_info values
+
 add hooks into transport base class for checking & updating a result set cache
    ie via a standard cache interface such as:
    http://search.cpan.org/~robm/Cache-FastMmap/FastMmap.pm
@@ -1278,7 +1130,9 @@ in such a way that appropriate http cache headers are added to the results
 so that web caches (squid etc) could be used to implement the caching.
 (MUST require the use of GET rather than POST requests.)
 
-Rework handling of installed_methods to not piggyback on dbh_attributes?
+Neat way for $h->trace to enable transport tracing.
+
+Rework handling of installed_methods to not piggback on dbh_attributes?
 
 Perhaps support transactions for transports where it's possible (ie null and stream)?
 Would make stream transport (ie ssh) more useful to more people.
@@ -1287,6 +1141,6 @@ Make sth_result_attr more like dbh_attributes (using '*' etc)
 
 Add @val = FETCH_many(@names) to DBI in C and use in Gofer/Execute?
 
-Implement _new_sth in C.
+Implement DBI::st::TIEHASH etc in C.
 
 =cut
