@@ -1,6 +1,6 @@
 package DBD::Gofer::Transport::Base;
 
-#   $Id: Base.pm 9866 2007-08-22 16:32:32Z timbo $
+#   $Id: Base.pm 9478 2007-05-01 11:03:38Z timbo $
 #
 #   Copyright (c) 2007, Tim Bunce, Ireland
 #
@@ -12,13 +12,12 @@ use warnings;
 
 use base qw(DBI::Gofer::Transport::Base);
 
-our $VERSION = sprintf("0.%06d", q$Revision: 9866 $ =~ /(\d+)/o);
+our $VERSION = sprintf("0.%06d", q$Revision: 9478 $ =~ /(\d+)/o);
 
 __PACKAGE__->mk_accessors(qw(
     trace
     go_dsn
     go_url
-    go_policy
     go_timeout
     go_retry_hook
     go_retry_limit
@@ -28,15 +27,9 @@ __PACKAGE__->mk_accessors_using(make_accessor_autoviv_hashref => qw(
 ));
 
 
+
 sub _init_trace { $ENV{DBD_GOFER_TRACE} || 0 }
 
-
-sub new_response {
-    my $self = shift;
-    return DBI::Gofer::Response->new(@_);
-}
-
-
 sub transmit_request {
     my ($self, $request) = @_;
     my $to = $self->go_timeout;
@@ -58,7 +51,7 @@ sub transmit_request {
         if ($@) {
             return $self->transport_timedout("transmit_request", $to)
                 if $@ eq "TIMEOUT\n";
-            return $self->new_response({ err => 1, errstr => $@ });
+            return DBI::Gofer::Response->new({ err => 1, errstr => $@ });
         }
 
         return $response;
@@ -100,7 +93,7 @@ sub receive_response {
         if ($@) {
             return $self->transport_timedout("receive_response", $to)
                 if $@ eq "TIMEOUT\n";
-            return $self->new_response({ err => 1, errstr => $@ });
+            return DBI::Gofer::Response->new({ err => 1, errstr => $@ });
         }
         return $response;
     };
@@ -135,7 +128,7 @@ sub response_needs_retransmit {
 
     if (not defined $retry) {
         my $errstr = $response->errstr || '';
-        $retry = 1 if $errstr =~ m/induced by DBI_GOFER_RANDOM/;
+        $retry = 1 if $errstr =~ m/fake error induced by DBI_GOFER_RANDOM_FAIL/;
     }
 
     if (not defined $retry) {
@@ -148,13 +141,9 @@ sub response_needs_retransmit {
         return 0;
     }
 
-    my $retry_limit = $self->go_retry_limit;
-    # $retry_limit = 2 unless defined $retry_limit; # not safe enough to do this
-    if (!$retry_limit) {
-        $self->trace_msg("response_needs_retransmit: retries disabled (retry_limit not set)\n");
-        return 0;
-    }
     my $meta = $request->meta;
+    my $retry_limit = $self->go_retry_limit;
+    $retry_limit = 2 unless defined $retry_limit;
     if (($meta->{retry_count}||=0) >= $retry_limit) {
         $self->trace_msg("response_needs_retransmit: $meta->{retry_count} is too many retries\n");
         return 0;
@@ -169,7 +158,7 @@ sub response_needs_retransmit {
 sub transport_timedout {
     my ($self, $method, $timeout) = @_;
     $timeout ||= $self->go_timeout;
-    return $self->new_response({ err => 1, errstr => "DBD::Gofer $method timed-out after $timeout seconds" });
+    return DBI::Gofer::Response->new({ err => 1, errstr => "DBD::Gofer $method timed-out after $timeout seconds" });
 }
 
 
@@ -179,92 +168,18 @@ sub transport_timedout {
 
 DBD::Gofer::Transport::Base - base class for DBD::Gofer client transports
 
-=head1 SYNOPSIS
 
-  my $remote_dsn = "..."
-  DBI->connect("dbi:Gofer:transport=...;url=...;timeout=...;retry_limit=...;dsn=$remote_dsn",...)
+=head1 AUTHOR AND COPYRIGHT
 
-or, enable by setting the DBI_AUTOPROXY environment variable:
+The DBD::Gofer, DBD::Gofer::* and DBI::Gofer::* modules are
+Copyright (c) 2007 Tim Bunce. Ireland.  All rights reserved.
 
-  export DBI_AUTOPROXY='dbi:Gofer:transport=...;url=...'
-
-which will force I<all> DBI connections to be made via that Gofer server.
-
-=head1 DESCRIPTION
-
-This is the base class for all DBD::Gofer client transports.
-
-=head1 ATTRIBUTES
-
-Gofer transport attributes can be specified either in the attributes parameter
-of the connect() method call, or in the DSN string. When used in the DSN
-string, attribute names don't have the C<go_> prefix.
-
-=head2 go_dsn
-
-The full DBI DSN that the Gofer server should connect to on your behalf.
-
-When used in the DSN it must be the last element in the DSN string.
-
-=head2 go_timeout
-
-A time limit for sending a request and receiving a response. Some drivers may
-implement sending and receiving as separate steps, in which case (currently)
-the timeout applies to each separately.
-
-If a request needs to be resent then the timeout is restarted for each sending
-of a request and receiving of a response.
-
-=head2 go_retry_limit
-
-The maximum number of times an request may be retried. The default is 2.
-
-=head2 go_retry_hook
-
-This subroutine reference is called, if defined, for each response received where $response->err is true.
-
-The subroutine is pass three parameters: the request object, the response object, and the transport object.
-
-If it returns an undefined value then the default retry behaviour is used. See L</RETRY ON ERROR> below.
-
-If it returns a defined but false value then the request is not resent.
-
-If it returns true value then the request is resent, so long as the number of retries does not exceed C<go_retry_limit>.
-
-=head1 RETRY ON ERROR
-
-The default retry on error behaviour is:
-
- - Retry if the error was due to DBI_GOFER_RANDOM. See L<DBI::Gofer::Execute>.
-
- - Retry if $request->is_idempotent returns true. See L<DBI::Gofer::Request>.
-
-A retry won't be allowed if the number of previous retries has reached C<go_retry_limit>.
-
-=head1 TRACING
-
-Tracing of gofer requests and reponses can be enabled by setting the
-C<DBD_GOFER_TRACE> environment variable. A value of 1 gives a reasonably
-compact summary of each request and response. A value of 2 or more gives a
-detailed, and voluminous, dump.
-
-The trace is written using DBI->trace_msg() and so is written to the default
-DBI trace output, which is usually STDERR.
-
-=head1 AUTHOR
-
-Tim Bunce, L<http://www.linkedin.com/in/timbunce>
-
-=head1 LICENCE AND COPYRIGHT
-
-Copyright (c) 2007, Tim Bunce, Ireland. All rights reserved.
-
-This module is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself. See L<perlartistic>.
+You may distribute under the terms of either the GNU General Public License or
+the Artistic License, as specified in the Perl README file.
 
 =head1 SEE ALSO
 
-L<DBD::Gofer>, L<DBI::Gofer::Request>, L<DBI::Gofer::Response>, L<DBI::Gofer::Execute>.
+L<DBD::Gofer>
 
 and some example transports:
 
