@@ -219,13 +219,21 @@ package DBD::Proxy::db; # ====== DATABASE ======
 
 $DBD::Proxy::db::imp_data_size = 0;
 
-# XXX probably many more methods need to be added here.
+# XXX probably many more methods need to be added here
+# in order to trigger our AUTOLOAD to redirect them to the server.
+# (Unless the sub is declared it's bypassed by perl method lookup.)
 # See notes in ToDo about method metadata
+# The question is whether to add all the methods in %DBI::DBI_methods
+# to the corresponding classes (::db, ::st etc)
+# Also need to consider methods that, if proxied, would change the server state
+# in a way that might not be visible on the client, ie begin_work -> AutoCommit.
+
 sub commit;
 sub connected;
 sub rollback;
 sub ping;
 
+
 use vars qw(%ATTR $AUTOLOAD);
 
 # inherited: STORE / FETCH against this class.
@@ -249,36 +257,31 @@ use vars qw(%ATTR $AUTOLOAD);
 sub AUTOLOAD {
     my $method = $AUTOLOAD;
     $method =~ s/(.*::(.*)):://;
-    # warn "AUTOLOAD of $method";
     my $class = $1;
     my $type = $2;
-    my %expand =
-	( 'method' => $method,
-	  'class' => $class,
-	  'type' => $type,
-	  'h' => "DBI::_::$type"
-	);
-    my $method_code = UNIVERSAL::can($expand{'h'}, $method) ?
-	q/package ~class~;
-          sub ~method~ {
+    #warn "AUTOLOAD of $method (class=$class, type=$type)";
+    my %expand = (
+        'method' => $method,
+        'class' => $class,
+        'type' => $type,
+        'call' => "$method(\@_)",
+        # XXX was trying to be smart but was tripping up over the DBI's own
+        # smartness. Disabled, but left here in case there are issues.
+    #   'call' => (UNIVERSAL::can("DBI::_::$type", $method)) ? "$method(\@_)" : "func(\@_, '$method')",
+    );
+
+    my $method_code = q{
+        package ~class~;
+        sub ~method~ {
             my $h = shift;
-	    my @result = wantarray
-		? eval {        $h->{'proxy_~type~h'}->~method~(@_) }
-		: eval { scalar $h->{'proxy_~type~h'}->~method~(@_) };
+            local $@;
+            my @result = wantarray
+                ? eval {        $h->{'proxy_~type~h'}->~call~ }
+                : eval { scalar $h->{'proxy_~type~h'}->~call~ };
             return DBD::Proxy::proxy_set_err($h, $@) if $@;
-            wantarray ? @result : $result[0];
-          }
-	 / :
-        q/package ~class~;
-	  sub ~method~ {
-	    my $h = shift;
-	    my @result = wantarray
-		? eval {        $h->{'proxy_~type~h'}->func(@_, '~method~') }
-		: eval { scalar $h->{'proxy_~type~h'}->func(@_, '~method~') };
-	    return DBD::Proxy::proxy_set_err($h, $@) if $@;
-	    wantarray ? @result : $result[0];
-          }
-         /;
+            return wantarray ? @result : $result[0];
+        }
+    };
     $method_code =~ s/\~(\w+)\~/$expand{$1}/eg;
     local $SIG{__DIE__} = 'DEFAULT';
     my $err = do { local $@; eval $method_code.2; $@ };
@@ -304,6 +307,7 @@ sub disconnect ($) {
     # Drop database connection at remote end
     my $rdbh = $dbh->{'proxy_dbh'};
     local $SIG{__DIE__} = 'DEFAULT';
+    local $@;
     eval { $rdbh->disconnect() };
     DBD::Proxy::proxy_set_err($dbh, $@) if $@;
     
@@ -892,6 +896,26 @@ method will be used (which will be faster but may be wrong).
 
 =head1 KNOWN ISSUES
 
+=head2 Unproxied method calls
+
+If a method isn't being proxied, try declaring a stub sub in the appropriate
+package (DBD::Proxy::db for a dbh method, and DBD::Proxy::st for an sth method).
+For example:
+
+    sub DBD::Proxy::db::selectall_arrayref;
+
+That will enable selectall_arrayref to be proxied.
+
+Currently many methods aren't explicitly proxied and so you get the DBI's
+default methods executed on the client.
+
+Some of those methods, like selectall_arrayref, may then call other methods
+that are proxied (selectall_arrayref calls fetchall_arrayref which calls fetch
+which is proxied). So things may appear to work but operate more slowly than
+the could.
+
+This may all change in a later version.
+
 =head2 Complex handle attributes
 
 Sometimes handles are having complex attributes like hash refs or
