@@ -1,6 +1,6 @@
 package DBI::Gofer::Transport::stream;
 
-#   $Id: stream.pm 12536 2009-02-24 22:37:09Z timbo $
+#   $Id: stream.pm 9497 2007-05-03 16:21:37Z timbo $
 #
 #   Copyright (c) 2007, Tim Bunce, Ireland
 #
@@ -15,7 +15,7 @@ use DBI::Gofer::Execute;
 
 use base qw(DBI::Gofer::Transport::pipeone Exporter);
 
-our $VERSION = sprintf("0.%06d", q$Revision: 12536 $ =~ /(\d+)/o);
+our $VERSION = sprintf("0.%06d", q$Revision: 9497 $ =~ /(\d+)/o);
 
 our @EXPORT = qw(run_stdio_hex);
 
@@ -46,7 +46,7 @@ sub run_stdio_hex {
 
         # there's no way to access the stats currently
         # so this just serves as a basic test and illustration of update_stats()
-        $executor->update_stats($request, $response, $frozen_request, $frozen_response, $time_received, 1);
+        $executor->update_stats($request, $response, $frozen_request, $frozen_response, $time_received);
     }
     DBI->trace_msg("$0 ending (pid $$)\n");
 }
@@ -55,22 +55,21 @@ sub run_stdio_hex {
 __END__
 
 =head1 NAME
-
+    
 DBI::Gofer::Transport::stream - DBD::Gofer server-side transport for stream
-
+    
 =head1 SYNOPSIS
 
 See L<DBD::Gofer::Transport::stream>.
 
-=head1 AUTHOR
+=head1 AUTHOR AND COPYRIGHT
 
-Tim Bunce, L<http://www.tim.bunce.name>
+The DBD::Gofer, DBD::Gofer::* and DBI::Gofer::* modules are
+Copyright (c) 2007 Tim Bunce. Ireland.  All rights reserved.
 
-=head1 LICENCE AND COPYRIGHT
+You may distribute under the terms of either the GNU General Public License or
+the Artistic License, as specified in the Perl README file.
 
-Copyright (c) 2007, Tim Bunce, Ireland. All rights reserved.
-
-This module is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself. See L<perlartistic>.
 
 =cut
+
